/* ifs.cpp
 *
 * ifs.cpp
 *
 */


#include "window.h"
#include "view.h"
#include "bitmap.h"
#include "ifs.h"
 
mainview *mview;
mainwindow *mwindow;
BRect wrect, vrect;
sem_id ifssem;

ifsapp::ifsapp()
	:BApplication('bifs')
{
	SetPulseRate(100);
}

void ifsapp::AboutRequested(void)
{
	BAlert *aboutalert;
	long res;

#ifdef DEBUG
	printf("about\n");
#endif
	// show info alert	
	aboutalert = new BAlert(
		"", 
		"IFStoy  Stig.A.Olsen@hda.hydro.com\nA small Be app for making IFS images\n",
		"Okay, dude",
		NULL,
		NULL,
		B_WIDTH_FROM_WIDEST,
		B_INFO_ALERT);
			
	aboutalert->SetShortcut(0, B_ESCAPE);
	res = aboutalert->Go();
	
}

void ifsapp::Pulse(void)
{
	static int errorcnt = 0;
	int i, j;
	float nx, ny, x ,y;
	float rnd, prob;
	int irnd;
	int xpos, ypos;
	
#ifdef DEBUG
//	printf("app pulse\n");
#endif
	// draw pixels in bitmap

	acquire_sem(ifssem);

	x = 0.0; y = 0.0;
	
	for(i = 0; i < ifs.iter; i++) {
		irnd = rand() % 10000;
		rnd = (float) irnd / 10000.0;		
		prob = ifs.ifstable[0][6];
		j = 0;
		while(prob < rnd) {
			j++;
			prob = prob + ifs.ifstable[j][6];
		}
		if(j == ifs.entries) {
			errorcnt++;
			printf("ERROR\tprob=%f\trnd=%f\terrorcnt=%d\n", prob, rnd, errorcnt);
			if(errorcnt > 50) be_app->Quit();
		}
		nx = ifs.ifstable[j][0]*x + ifs.ifstable[j][1]*y + ifs.ifstable[j][4];
		ny = ifs.ifstable[j][2]*x + ifs.ifstable[j][3]*y + ifs.ifstable[j][5];
		if(i > 10) {
			xpos = (int) ((x + ifs.xoffset) * ifs.windowx / ifs.width);
			ypos = (int) ((y + ifs.yoffset) * ifs.windowy / ifs.height);
			if(ifs.reverse) bitplot(xpos, ifs.windowy - ypos, ifs.color);
//				ifs.color + ifs.numcolors * i/ifs.iter);
			else bitplot((float) xpos, (float) ypos, ifs.color);
		}
		x = nx; y = ny;
	}

	release_sem(ifssem);

	// invalidate view
}



void ifsapp::ArgvReceived(int argc, char **argv)
{
	int i;
		
#ifdef DEBUG
	printf("arguments:\n");
#endif	
	
	for(i = 0; i < argc; i++) {
#ifdef DEBUG
		printf("%s\n", argv[i]);
#endif
	}
}

void ifsapp::ReadyToRun(void)
{
	// setup main menu
	BMenuItem *item;
	BPopUpMenu *menu = new BPopUpMenu("", FALSE, FALSE);

	item = new BMenuItem("About IFSToy", new BMessage(B_ABOUT_REQUESTED));
	item->SetTarget(be_app);
	menu->AddItem(item);
	
	item = new BMenuItem("Quit", new BMessage(B_QUIT_REQUESTED));
	item->SetTarget(be_app);
	menu->AddItem(item);

	be_app->SetMainMenu(menu);

	// setup offscreen bitmap
	initbitmap(ifs.screenx,ifs.screeny);
	vrect.Set(0.0, 0.0, ifs.screenx, ifs.screeny);
	mview = new mainview(vrect, "ifsview", B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED, bitptr());
	
	// setup window with view
	wrect.Set(50.0, 50.0, 50.0+ifs.windowx, 50.0+ifs.windowy);
	mwindow = new mainwindow(wrect, "IFStoy", B_TITLED_WINDOW, NULL);
	mwindow->AddChild(mview);
	mwindow->Show();
}

void ifsapp::RefsReceived(BMessage *message)
{
}

bool ifsapp::QuitRequested(void)
{
	mwindow->Quit();
	removebitmap();
	Quit();
	return TRUE;
}

void setdefaults(void) 
{
	// default ifs values
	ifs.windowx = 300;
	ifs.windowy = 300;
	ifs.iter = 5000;
	ifs.color = 52;
	ifs.numcolors = 10;
	ifs.reverse = 1;
	ifs.width = 7.0;
	ifs.height = 12.0;
	ifs.xoffset = 3.0;
	ifs.yoffset = 0.0;
	ifs.screenx = 1024;
	ifs.screeny = 768;
	ifs.entries = 4;

	ifs.ifstable[0][0] = 0.0;
	ifs.ifstable[0][1] = 0.0;
	ifs.ifstable[0][2] = 0.0;
	ifs.ifstable[0][3] = 0.16;
	ifs.ifstable[0][4] = 0.0;
	ifs.ifstable[0][5] = 0.0;
	ifs.ifstable[0][6] = 0.01;
	
	ifs.ifstable[1][0] = 0.85;
	ifs.ifstable[1][1] = 0.04;
	ifs.ifstable[1][2] = -0.04;
	ifs.ifstable[1][3] = 0.85;
	ifs.ifstable[1][4] = 0.0;
	ifs.ifstable[1][5] = 1.6;
	ifs.ifstable[1][6] = 0.85;
	
	ifs.ifstable[2][0] = 0.2;
	ifs.ifstable[2][1] = -0.26;
	ifs.ifstable[2][2] = 0.23;
	ifs.ifstable[2][3] = 0.22;
	ifs.ifstable[2][4] = 0.0;
	ifs.ifstable[2][5] = 1.6;
	ifs.ifstable[2][6] = 0.07;
	
	ifs.ifstable[3][0] = -0.15;
	ifs.ifstable[3][1] = 0.28;
	ifs.ifstable[3][2] = 0.26;
	ifs.ifstable[3][3] = 0.24;
	ifs.ifstable[3][4] = 0.0;
	ifs.ifstable[3][5] = 0.44;
	ifs.ifstable[3][6] = 0.07;
}

int main(int argc, char **argv, char **env)
{
	ifsapp *myapp;

	setdefaults();
	srand(0x342ffe7);

	ifssem = create_sem(1, "ifs semaphore");

	myapp = new ifsapp();
	
	myapp->Run();
	
	delete myapp;

	delete_sem(ifssem);

	return 0;
}

