/* view.cpp
 *
 * implementation of view functions  
 */

#include "view.h"

smartFile::smartFile()
	: BFile()
{
	bufferpos = 0;
	buffer = (char *) NULL;
}

smartFile::~smartFile()
{
	CloseData();
	free(buffer);
}

int smartFile::readFile() 
{
	int size;
	long rl;
	int i;
	
	free(buffer);
	buffer = (char *) malloc(DataSize());
	if(buffer == (char *) NULL) {
		printf("couldn't allocate memory for buffer\n");
	}
	OpenData();
	rl = Read((void *) buffer, DataSize());
	if(rl != DataSize()) {
		printf("read length, datasize mismatch, %d != %d\n",
			 rl, DataSize());
	}
#ifdef DEBUG
	printf("buffer: \n");
	for(i = 0; i < DataSize(); i++)
		printf("%c", buffer[i]);
#endif
	bufferpos = 0; // set at start of buffer
	return rl;		
}

int smartFile::getInt()
{
	int number;
	int negative;
	
	number = 0;
	negative = FALSE;
	// drop white space
	while(buffer[bufferpos] < '0' || buffer[bufferpos] > '9') bufferpos++;
	// check negative
	if(bufferpos > 0) {
		if(buffer[bufferpos-1] == '-') negative = TRUE;
	}
	// add first digit
	number = buffer[bufferpos] - '0';
	bufferpos++;
	// add next digits
	while(buffer[bufferpos] >= '0' && buffer[bufferpos] <= '9') {
		number *= 10;
		number += buffer[bufferpos] - '0';
		bufferpos++;
	} 
#ifdef DEBUG
	printf("returning %d, bufferpos = %d\n", number, bufferpos);	
#endif
	if(negative) number = -number;
	return number;	
}

float smartFile::getFloat()
{
	float real, fract;
	int negative;
	
	real = 0.0;
	fract = 10.0;
	negative = FALSE;
	// drop white space
	while(buffer[bufferpos] < '0' || buffer[bufferpos] > '9') bufferpos++;
	// check negative
	if(bufferpos > 0) {
		if(buffer[bufferpos-1] == '-') negative = TRUE;
	}
	// add first digit
	real += (float) (buffer[bufferpos] - '0');
	bufferpos++;
	// add next digits
	while(buffer[bufferpos] >= '0' && buffer[bufferpos] <= '9') {
		real *= 10.0;
		real += (float) (buffer[bufferpos] - '0');
		bufferpos++;
	}
	// check for fractional part
	if(buffer[bufferpos] == '.') {
		bufferpos++;
		while(buffer[bufferpos] >= '0' && buffer[bufferpos] <= '9') {
			real = real + (buffer[bufferpos] - '0') / fract;
			fract *= 10.0;
			bufferpos++;
		} 	
	}		
	if(negative) real = -real;
#ifdef DEBUG
	printf("returning (float) %f, bufferpos = %d\n", real, bufferpos);
#endif
	return (float) real;
}

int smartFile::fillSem()
{
	int i,j;
	
	// check validity of file
	if(buffer[0] != '#' ||
	  buffer[1] != 'i' ||
	  buffer[2] != 'f' ||
	  buffer[3] != 's') {
#ifdef DEBUG
		printf("this is not an ifs datafile\n");
#endif
		return FALSE;
	} else {
#ifdef DEBUG
		printf("this is an ifs datafile\n");
#endif
	}
	
	ifs.width = getFloat();
	ifs.height = getFloat();
	ifs.xoffset = getFloat();
	ifs.yoffset = getFloat();
	ifs.entries = getInt();
	ifs.reverse = getInt();
#ifdef DEBUG		
	printf("width, height, xoffset, yoffset, entries\n%f, %f, %f, %f, %d\n",
		ifs.width, ifs.height, ifs.xoffset, ifs.yoffset, ifs.entries);
	
	printf("table\n");
#endif
	for(i = 0; i < ifs.entries; i++) {
		for(j = 0; j < 7; j++) {
			ifs.ifstable[i][j] = getFloat();
//			printf("%f ", ifs.ifstable[i][j]);
		}
#ifdef DEBUG
		printf("\n");
#endif	
	}	
	
	return TRUE;
}

mainview::mainview(BRect frame, const char *name, ulong resizemode, ulong flags, BBitmap *bmap)
	:BView(frame, name, resizemode, flags)
{
	image = bmap;	
}

void mainview::Draw(void)
{
	BPoint point(0.0, 0.0);

	DrawBitmap(image, point);
	Sync();
}

void mainview::Pulse(void)
{
	Draw();
	Sync();
}

bool mainview::MessageDropped(BMessage *bmess, BPoint point, BPoint offset)
{
	record_ref ifsref;
	smartFile *ifsfile;
	long counter;
	long countFound;
	ulong typeFound;
	char buffer[256];
	BAlert *infoalert;
	long res;

//	// show info alert	
//	infoalert = new BAlert(
//		"", 
//		"Can't handle drag'n'drop yet",
//		"Okay, dude",
//		NULL,
//		NULL,
//		B_WIDTH_FROM_WIDEST,
//		B_INFO_ALERT);
//			
//	infoalert->SetShortcut(0, B_ESCAPE);
//	res = infoalert->Go();	
//	
	// no message or no refs
	if(!bmess->GetInfo("refs", &typeFound, &countFound))
		return TRUE;
	if(countFound < 1)
		return TRUE;

	for(counter = 0; counter < countFound; counter++) {
		ifsref = bmess->FindRef("refs", counter);
		if(does_ref_conform(ifsref, "File")) {
			acquire_sem(ifssem);
			ifsfile = new smartFile();
			ifsfile->SetRef(ifsref);
			ifsfile->readFile();	
			clearbitmap(0);
			ifsfile->fillSem();
			delete ifsfile;
			release_sem(ifssem);
		} else {
			printf("Couldn't open data!\n");
		}			
	}
			

	return TRUE;
}
 
